/*************************************************
*     Exim - an Internet mail transport agent    *
*************************************************/

/* Copyright (c) University of Cambridge 1995 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

/* This file is the header that is the only Exim header to be included in the
source for the local_scan.c() function. It contains definitions that are made
available for use in that function, and which are documented. */


/* Some basic types that make some things easier, and the store functions. */

#include <sys/types.h>
#include "mytypes.h"
#include "store.h"


/* The function and its return codes. */

extern int local_scan(int, uschar **);

enum { LOCAL_SCAN_ACCEPT, LOCAL_SCAN_REJECT, LOCAL_SCAN_TEMPREJECT };


/* Available logging destinations */

#define LOG_MAIN        1    /* Write to the main log */
#define LOG_REJECT     16    /* Write to the reject log, with headers */


/* Structure definitions that are documented as visible in the function. */

typedef struct header_line {
  struct header_line *next;
  int    type;
  int    slen;
  uschar *text;
} header_line;

/*Structure for holding information about an envelope address. The errors_to
field is always NULL except for one_time aliases that had errors_to on the
routers that generated them. */

typedef struct recipient_item {
  uschar *address;              /* the recipient address */
  int     pno;                  /* parent number for "one_time" alias, or -1 */
  uschar *errors_to;            /* the errors_to address or NULL */
} recipient_item;


/* Global variables that are documented as visible in the function. */

extern header_line   *header_last;     /* Final header */
extern header_line   *header_list;     /* First header */

extern uschar *interface_address;      /* Interface for incoming call */
extern int     interface_port;         /* Port number for incoming call */

extern uschar *received_protocol;      /* Name of incoming protocol */
extern int     recipients_count;       /* Number of recipients */
extern recipient_item *recipients_list;/* List of recipient addresses */

extern unsigned char *sender_address;  /* Sender address */

extern uschar *sender_host_address;    /* IP address of sender, as chars */
extern uschar *sender_host_authenticated; /* Name of authentication mechanism */
extern uschar *sender_host_name;       /* Host name from lookup */
extern int     sender_host_port;       /* Port number of sender */



/* Functions that are documented as visible in the function. */

extern int     child_close(pid_t, int);
extern pid_t   child_open_exim(int *);
extern uschar *expand_string(uschar *);
extern void    header_add(int, char *, ...);
extern void    log_write(unsigned int, int, char *format, ...);
extern void    receive_add_recipient(uschar *, int);
extern uschar *string_copy(uschar *);
extern uschar *string_copyn(uschar *, int);
extern uschar *string_sprintf(char *, ...);


/* End of local_scan.h */
