/*************************************************
*     Exim - an Internet mail transport agent    *
*************************************************/

/* Copyright (c) University of Cambridge 1995 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */


#include "../exim.h"
#include "rf_functions.h"



/*************************************************
*          Change domain in an address           *
*************************************************/

/* When a router wants to change the address that is being routed, it must set
new "active" field in the address. If the headers are to be rewritten, the new
domain must be placed in "domain", and the relevant flag set. In normal cases,
the "active" and "rcpt" fields are the same, and both must be changed. Also the
"unique" field, which has the same domain as "rcpt" (but this is empty when
verifying). However, after masquerading, the "rcpt" field is different, and if
this is the case, neither it nor "unique" is changed, and header rewriting is
not done. We pick up the local part from the "active" field so as to get it in
external form - caseful, and with any quoting retained.

Arguments:
  addr        the address block
  domain      the new domain
  rewrite     TRUE if headers lines are to be rewritten

Returns:      nothing
*/

void
rf_change_domain(address_item *addr, uschar *domain, BOOL rewrite)
{
uschar *at = Ustrrchr(addr->active, '@');
uschar *new = string_sprintf("%.*s@%s", at - addr->active, addr->active,
  domain);

if (addr->rcpt == addr->active)
  {
  addr->rcpt = new;
  if (rewrite)
    {
    addr->domain = domain;
    setflag(addr, af_rewrite_headers);
    }
  if (addr->unique != NULL)
    {
    at = Ustrrchr(addr->unique, '@');
    addr->unique = string_sprintf("%.*s@%s", at - addr->unique, addr->unique,
      domain);
    }
  }
addr->active = new;
}

/* End of rf_change_domain.c */
